# Model of Early Drop-Out
# David Dowdy
# Revision date: June 2, 2013

############################################################################
# DEFINE USER PARAMETERS
# Entries & TB transmission
extrapars <-c(TP = 100000,        # total population size
              B_a = 0.00011062,   # transmission rate
        		  Bx_hiv = 3.671,     # relative risk of infection if HIV+
              Bx_smn = 0.15,      # relative transmission rate of smear-neg cases (incl EPTB)
              LP = 0.5)           # protection against reinfection if latently infected

# Initial population -> make sure this agrees w/ extrapars
total_pop <- 100000    # size of initial population
latent_prot <- 0.5     # protection of latent TB against reinfection
transmit <- 0.00011062 # transmission rate, fit to TB incidence
trans_hiv <- 3.671     # relative risk of infection if HIV+
trans_smn <-0.15       # relative transmission rate of smear-neg cases (incl EPTB)
pop_inc <-0.0225       # rate of population increase, fit to pop size

# TB progression
fast_prop <- 0.14      # proportion of fast progression, HIV-neg
fast_prop_h <- 0.4     # fast progression, HIV-pos
lpslow <- 0.00058      # slow progression of latent TB, HIV-neg
lpslow_h <- 0.0331     # slow progression of latent TB, HIV-pos
smpos_prop <- 0.65     # proportion of TB that is smear-positive, HIV-neg
smpos_prop_h <- 0.5    # propotion of TB that is smear-positive, HIV-pos

# Mortality
m_notb <- 1/46.5       # mortality, no TB or HIV
m_h <- 0.053           # mortality from HIV
m_tb_sp <- 0.231       # mortality from TB, no HIV, smear-pos
m_tb_sn <- m_tb_sp*0.286  # mortality from TB, no HIV, smear-neg
m_tbh <- 2.0           # mortality from TB, HIV

# Self Cure
cure_sp <-0.099        # self-cure rate, smear-pos
cure_sn <-0.264        # self-cure rate, smear-neg

# HIV
hiv_inc <- 0.0047813    # annual incidence of HIV

# TB Diagnosis & Treatment
init_def <- 0.15 		    # initial default, modifiable by same-day microscopy
sm_sens <- 62/65 		    # sensitivity of smear strategy
empiric_sens <- 0.254   # empiric treatment sensitivity (Wilkinson, IJTLD 2000; 4:513)
empiric_sensn <- empiric_sens   # empiric treatment sensitivity, smear-neg
tx_suc <- 0.8  		      # treatment success proportion
dx_rate <- 0.80    	    # rate of seeking care, no HIV
dx_rate_h <- 18.07      # rate of seeking care, HIV
rlrate <-0.000          # relapse rate -> set to zero based on model fit

tx_sp <- dx_rate*tx_suc*(sm_sens*(1-init_def)+ 
         (1-sm_sens*(1-init_def))*empiric_sens)      # treatment rate, no HIV, smear-pos
txh_sp <- dx_rate_h*tx_suc*(sm_sens*(1-init_def)+ 
         (1-sm_sens*(1-init_def))*empiric_sens)      # treatment rate, HIV, smear-pos
tx_sn <- dx_rate*tx_suc*empiric_sensn  		           # treatment rate, no HIV, smear-neg
txh_sn <- dx_rate_h*tx_suc*empiric_sensn 		         # treatment rate, HIV, smear-neg


# Relapse
rl <- rlrate + lpslow     # rate of relapse, treated TB
rlh <- rlrate + lpslow_h  # rate of relapse, treated TB, HIV


#####################################################################
# Set initial matrices and population size
# Reminder of the Matrix Structure:
# matrix = 5 x 2
# rows: 5 TB states (1=S, 2=L, 3=Ai, 4=An, 5=T)
# cols: 2 HIV states (1=no, 2=yes) 

# Start all the various matrices off as empty:
statemat <- array(total_pop, dim=c(5,2)) # matrix of initial population sizes
mortmat <- array(0, dim=c(5,2))       # matrix of mortality rates
entrymat <- array(0, dim=c(5,2))      # matrix of entries into the population
mastermat <- array(0, dim=c(5,2,5,2)) # master transmission matrix, 1st 2 dimensions: loss, next 2:gain
outmat <- array(0, dim=c(2000,7))     # matrix of outputs

################################################
# Mortality
# matrix = 5 x 2
# rows: 5 TB states (1=S, 2=L, 3=Ai, 4=An, 5=T)
# cols: 2 HIV states (1=no, 2=yes) 
mortmat[,] <- -m_notb
mortmat[,2] <- mortmat[,2] - m_h
mortmat[3,1] <- mortmat[3,1] - m_tb_sp
mortmat[4,1] <- mortmat[4,1] - m_tb_sn
mortmat[3,2] <- mortmat[3,2] - m_tbh
mortmat[4,2] <- mortmat[4,2] - m_tbh

mortname<-paste("m",1:10,sep="")
mortvect <-c(mortmat)
names(mortvect)<-mortname

i <- rep(1:2,each=5)
j <- rep(1:5,2)
k <- array(c(j,i),dim=c(10,2))
m <- array(c(k,k),dim=c(10,4))
mastermat[m]<-mastermat[m]+mortvect


################################################
# TB Progression (Latent/fast-> Latent/slow-> Active)
# matrix = 5 x 2
# rows: 5 TB states (1=S, 2=L, 3=Ai, 4=An, 5=T)
# cols: 2 HIV states (1=no, 2=yes)

mastermat[2,1,3,1]<-lpslow*smpos_prop
mastermat[2,2,3,2]<-lpslow_h*smpos_prop_h
mastermat[2,1,4,1]<-lpslow*(1-smpos_prop)
mastermat[2,2,4,2]<-lpslow_h*(1-smpos_prop_h)



################################################
# Treatment and Relapse
# matrix = 5 x 2
# rows: 5 TB states (1=S, 2=L, 3=Ai, 4=An, 5=T)
# cols: 2 HIV states (1=no, 2=yes)

# Treatment:
mastermat[3,1,5,1]<-tx_sp + cure_sp
mastermat[3,2,5,2]<-txh_sp
mastermat[4,1,5,1]<-tx_sn + cure_sn
mastermat[4,2,5,2]<-txh_sn


# Relapse:
mastermat[5,1,3,1]<-rl*smpos_prop
mastermat[5,1,4,1]<-rl*(1-smpos_prop)
mastermat[5,2,3,2]<-rlh*smpos_prop_h
mastermat[5,2,4,2]<-rlh*(1-smpos_prop_h)


################################################
# HIV Infection
# matrix = 5 x 2
# rows: 5 TB states (1=S, 2=L, 3=Ai, 4=An, 5=T)
# cols: 2 HIV states (1=no, 2=yes)

# HIV infection & progression:
j1 <- array(c(1:5,rep(1,5),1:5,rep(2,5)),dim=c(5,4)) 
mastermat[j1]<-hiv_inc                                       
 

###################################################################
# Function for input into the differential equation solver:

# first, put mastermat into a useable format
mastername<-paste("M",1:100,sep="")  # 10x10 parameters
mastervect <-c(mastermat)
names(mastervect)<-mastername
mastervect<-c(mastervect,extrapars)

TBdx<-function(t, state, parameters) {
        x<-as.list(c(state,parameters))
        with(x,{
             statev <- unlist(x[1:10])
             masterv <- unlist(x[11:(11+10*10-1)])

             # force of infection
             force <- B_a*((S3+S8)+Bx_smn*(S4+S9))
            
             # now, create an "infection matrix"
             infectmat <- array(0, dim=c(5,2,5,2)) 
             kinf <- array(c(rep(c(1,1,1,2,2,5,5),2),rep(c(1,2),each=7),
             rep(c(2,3,4,3,4,3,4),2),rep(c(1,2),each=7)),dim=c(14,4))
             infectmat[kinf]<-force

             # adjust by prop rapid progression & prop infectious
             k1 <- array(c(c(1,2,5),rep(1,3),rep(3,3),rep(1,3)),dim=c(3,4))
             infectmat[k1]<-force*smpos_prop*fast_prop
             k2 <- array(c(c(1,2,5),rep(2,3),rep(3,3),rep(2,3)),dim=c(3,4))
             infectmat[k2]<-force*Bx_hiv*smpos_prop_h*fast_prop_h
             k3 <- array(c(c(1,2,5),rep(1,3),rep(4,3),rep(1,3)),dim=c(3,4))
             infectmat[k3]<-force*(1-smpos_prop)*fast_prop
             k4 <- array(c(c(1,2,5),rep(2,3),rep(4,3),rep(2,3)),dim=c(3,4))
             infectmat[k4]<-force*Bx_hiv*(1-smpos_prop_h)*fast_prop_h
             infectmat[1,1,2,1]<-force*(1-fast_prop)
             infectmat[1,2,2,2]<-force*Bx_hiv*(1-fast_prop_h)

             # add in protection from LTBI
             kli <- array(c(rep(c(2,5),2),rep(1,4),rep(rep(c(3,4),each=2)),rep(1,4)),dim=c(4,4))
             infectmat[kli]<-infectmat[kli]*(1-LP)

             # convert infectmat into a vector
             infectv<-c(infectmat)

             # add infectv to masterv
             masterv3<-masterv+infectv

             # create the change matrix
             mastermatr <-array(masterv3,dim=c(10,10))

             # sum deaths, to balance the population
             i5<-array(c(1:10),dim=c(10,2))
             i5a<-c(1:10)
             deaths<-sum(mastermatr[i5]*statev[i5a])
        		 mastermatr[i5]<-mastermatr[i5]+pop_inc

             # rate of change
             delta <- rep(0,10)
             delta[1] <- sum(mastermatr[,1]*statev) - 
                         sum(mastermatr[1,]*statev[1]) + 
                         mastermatr[1,1]*statev[1] -
                         deaths*0.6774 # 0.6774 is the proportion remaining uninfected at age 15
             delta[2] <- sum(mastermatr[,2]*statev) - 
                         sum(mastermatr[2,]*statev[2]) + 
                         mastermatr[2,2]*statev[2] -
                         deaths*(1-0.6774)
             delta[3] <- sum(mastermatr[,3]*statev) - 
                         sum(mastermatr[3,]*statev[3]) + 
                         mastermatr[3,3]*statev[3]
             delta[4] <- sum(mastermatr[,4]*statev) - 
                         sum(mastermatr[4,]*statev[4]) + 
                         mastermatr[4,4]*statev[4]
             delta[5] <- sum(mastermatr[,5]*statev) - 
                         sum(mastermatr[5,]*statev[5]) + 
                         mastermatr[5,5]*statev[5]
             delta[6] <- sum(mastermatr[,6]*statev) - 
                         sum(mastermatr[6,]*statev[6]) + 
                         mastermatr[6,6]*statev[6] 
             delta[7] <- sum(mastermatr[,7]*statev) - 
                         sum(mastermatr[7,]*statev[7]) + 
                         mastermatr[7,7]*statev[7]
             delta[8] <- sum(mastermatr[,8]*statev) - 
                         sum(mastermatr[8,]*statev[8]) + 
                         mastermatr[8,8]*statev[8]
             delta[9] <- sum(mastermatr[,9]*statev) - 
                         sum(mastermatr[9,]*statev[9]) + 
                         mastermatr[9,9]*statev[9]
             delta[10] <- sum(mastermatr[,10]*statev) - 
                          sum(mastermatr[10,]*statev[10]) + 
                          mastermatr[10,10]*statev[10]
             # return the rate of change
             list(c(delta))
             })
}


#  This function calculates outputs from a vector of population values
#  (statevect) and a vector of rates of change (mastervect)
outgen<-function(statevect, mastervect) {
        x<-as.list(c(statevect,mastervect))
        with(x,{
             statev <- unlist(x[1:10])
             masterv <- unlist(x[11:(11+10*10-1)])
             B_a<-transmit
             Bx_smn<-trans_smn
             Bx_hiv<-trans_hiv

             # force of infection, density-dependent
             force <- B_a*((S3+S8)+Bx_smn*(S4+S9))
            
             # now, create an "infection matrix"
             infectmat <- array(0, dim=c(5,2,5,2)) 
             kinf <- array(c(rep(c(1,1,1,2,2,5,5),2),rep(c(1,2),each=7),
             rep(c(2,3,4,3,4,3,4),2),rep(c(1,2),each=7)),dim=c(14,4))
             infectmat[kinf]<-force

             # adjust by prop rapid progression & prop infectious
             k1 <- array(c(c(1,2,5),rep(1,3),rep(3,3),rep(1,3)),dim=c(3,4))
             infectmat[k1]<-force*smpos_prop*fast_prop
             k2 <- array(c(c(1,2,5),rep(2,3),rep(3,3),rep(2,3)),dim=c(3,4))
             infectmat[k2]<-force*Bx_hiv*smpos_prop_h*fast_prop_h
             k3 <- array(c(c(1,2,5),rep(1,3),rep(4,3),rep(1,3)),dim=c(3,4))
             infectmat[k3]<-force*(1-smpos_prop)*fast_prop
             k4 <- array(c(c(1,2,5),rep(2,3),rep(4,3),rep(2,3)),dim=c(3,4))
             infectmat[k4]<-force*Bx_hiv*(1-smpos_prop_h)*fast_prop_h
             infectmat[1,1,2,1]<-force*(1-fast_prop)
             infectmat[1,2,2,2]<-force*Bx_hiv*(1-fast_prop_h)

             # add in protection from LTBI
             kli <- array(c(rep(c(2,5),4),rep(c(1,2),each=4),rep(rep(c(3,4),each=2),2),rep(c(1,2),each=4)),dim=c(8,4))
             infectmat[kli]<-infectmat[kli]*(1-LP)

             # convert infectmat into a vector
             infectv<-c(infectmat)

             # add infectv to masterv
             masterv3<-masterv+infectv

             # create the change matrix
             mastermatr <-array(masterv3,dim=c(10,10))

             # TB death matrix
             deathmatr <-array(0,dim=c(10,10))
             deathmatr[3,3]<-m_tb_sp
             deathmatr[4,4]<-m_tb_sn
             deathmatr[8,8]<-m_tbh
             deathmatr[9,9]<-m_tbh

             # sum deaths, to balance the population
             i5<-array(c(1:10),dim=c(10,2))
             i5a<-c(1:10)
             deaths<-sum(mastermatr[i5]*statev[i5a])

# Calculate TB mortality
i3aa<-c(3,4)
i3c<-array(c(3,4,3,4),dim=c(2,2))
TBmort <-sum(deathmatr[i3c]*statev[i3aa])

# Calculate HIV/TB mortality             
i4<-array(c(8,9),dim=c(2,2))
i5<-c(8,9)
HIVTBmort<-sum(deathmatr[i4]*statev[i5])

# Calculate TB incidence             
i7<-array(c(rep(c(1,2,5),2),rep(c(6,7,10),2),rep(c(3,4),each=3),rep(c(8,9),each=3)),dim=c(12,2))
incidence <-sum(mastermatr[i7]*statev[i7[,1]])

# Calculcate HIV/TB incidence             
i7hiv <-i7[7:12,]
hivtbinc <- sum(mastermatr[i7hiv]*statev[i7hiv[,1]])

# Calculate HIV prevalence             
i6<-c(seq(6,10))
HIVprev<-sum(statev[i6])/1000

# Calculate TB prevalence             
i6tb<-c(3,4,8,9)
TBprev<-sum(statev[i6tb])

# Function used internally to scale to the size of full African population             
i6pop<-c(seq(1,10))
pop<-sum(statev[i6pop])*691/100000

# Calculate HIV mortality             
i8<-array(c(seq(6,10)),dim=c(5,2))
HIVmort<-sum((deathmatr[i8]+m_h)*statev[i6])

incprevmort <- c(pop, incidence, hivtbinc, TBprev, HIVprev, TBmort, HIVTBmort)
incprevmort
})
}




##############################################################
# BASELINE = 2002
 infile<-(read.csv("jun30eq.csv"))         # this file contains the equilibrium population
 statevect <-infile[501,]                  # read in the equilibrium population (501st row)
 statevect <-(as.numeric(statevect[3:12]))
 statename<-paste("S",1:10,sep="")         # name the populations "S1", "S2", ..., "S10"
 names(statevect)<-statename

# Calculate years in time steps of 0.01 year
times <-seq(0,1,by=0.01)

library(deSolve)                           # package deSolve is required for this
out <- ode(statevect, times, TBdx, mastervect)  # calculate ODE's, then in next row generate outputs
for (i in 1:100) { outmat[i,] <- outgen(out[i,2:11],mastervect)
}
##############################################################

##############################################################
# FIT 2003 to 2010
# This has already been done, but the numbers are provided here for reference.

outmatm28 <-array(0,dim=c(2151,7))    # matrix for final output, baseline (will go into CSV)
outmatint <-array(0,dim=c(2151,7))    # matrix for final output, intervention (for CSV)
times2 <-seq(0,0.01,by=0.01)          # allows for calculation of timesteps of 0.01
outm28 <- ode(statevect, times, TBdx, mastervect)  # replication of "out" above

# These are the slopes of the fitting parameters that change over time, with initial guesses
pop_fact <- 6.136355e-06 # 0.00000178
Ba_fact <- 1.87653e-09 # 0.0000000040
hivtb_fact <- 0.001516318 # 0.00294
dx_fact <- 0.0004314459 # 0.000272
dxh_fact <-0.001118655 # 0.00258
hiv_fact <- 9.59718e-07
hiv_inc2 <- 0.003281713

# now repeat the same procedure as above, only at every 0.01-year timestep, recalculate the inputs
# do this for an 8 year (800-timestep) timeframe
for (abc in 1:800) { 

 statevect <-(as.numeric(outm28[2,2:11]))
 statename<-paste("S",1:10,sep="")
 names(statevect)<-statename

extrapars <-c(TP = 100000,                      # total population size
              B_a = 0.00011062-Ba_fact*(abc),   # transmission rate of TB
		  Bx_hiv = max(1,3.671-hivtb_fact*abc),     # relative risk of infection if HIV+
              Bx_smn = 0.15,                    # relative infectivity of smear-neg cases
              LP = 0.50)                        # protection against reinfection if latently infected

total_pop <- 100000                     # size of initial population
pop_inc <-0.02319+pop_fact*abc          # rate of population increase
transmit <- 0.00011062-Ba_fact*(abc)    # transmission rate of TB
trans_hiv <- max(1,3.671-hivtb_fact*abc)  # relative transmission of HIV-associated TB
hiv_inc <- hiv_inc2-hiv_fact*abc        # HIV incidence rate
init_def <- 0.15 	                      # initial default proportion
dx_rate <- 0.80+dx_fact*abc             # TB diagnosis rate, HIV-neg
dx_rate_h <- 18.07-dxh_fact*abc         # TB diagnosis rate, HIV-pos

tx_sp <- dx_rate*tx_suc*(sm_sens*(1-init_def)+ 
         (1-sm_sens*(1-init_def))*empiric_sens)     # treatment rate, no HIV, smear-pos
txh_sp <- dx_rate_h*tx_suc*(sm_sens*(1-init_def)+ 
         (1-sm_sens*(1-init_def))*empiric_sens)     # treatment rate, HIV, smear-pos
tx_sn <- dx_rate*tx_suc*empiric_sens  		          # treatment rate, no HIV, smear-neg
txh_sn <- dx_rate_h*tx_suc*empiric_sens 		        # treatment rate, HIV, smear-neg

mortmat <- array(0, dim=c(5,2))       # matrix of mortality rates
entrymat <- array(0, dim=c(5,2))      # matrix of entries into the population
mastermat <- array(0, dim=c(5,2,5,2)) # master transmission matrix, 1st 2 dimensions: loss, next 2:gain
mortmat[,] <- -m_notb                 # mortality occurs along the diagonal
mortmat[,2] <- mortmat[,2] - m_h      # HIV-associated mortality
mortmat[3,1] <- mortmat[3,1] - m_tb_sp # smear-pos TB mortality
mortmat[4,1] <- mortmat[4,1] - m_tb_sn # smear-neg TB mortality
mortmat[3,2] <- mortmat[3,2] - m_tbh   # TB-HIV mortality
mortmat[4,2] <- mortmat[4,2] - m_tbh

mortname<-paste("m",1:10,sep="")      # create a "mortality vector"
mortvect <-c(mortmat)
names(mortvect)<-mortname

# These rates of change are all cut-and-pasted from the corresponding equations above.
i <- rep(1:2,each=5)
j <- rep(1:5,2)
k <- array(c(j,i),dim=c(10,2))
m <- array(c(k,k),dim=c(10,4))
mastermat[m]<-mastermat[m]+mortvect
mastermat[2,1,3,1]<-lpslow*smpos_prop
mastermat[2,2,3,2]<-lpslow_h*smpos_prop_h
mastermat[2,1,4,1]<-lpslow*(1-smpos_prop)
mastermat[2,2,4,2]<-lpslow_h*(1-smpos_prop_h)
mastermat[3,1,5,1]<-tx_sp + cure_sp
mastermat[3,2,5,2]<-txh_sp
mastermat[4,1,5,1]<-tx_sn + cure_sn
mastermat[4,2,5,2]<-txh_sn
mastermat[5,1,3,1]<-rl*smpos_prop
mastermat[5,1,4,1]<-rl*(1-smpos_prop)
mastermat[5,2,3,2]<-rlh*smpos_prop_h
mastermat[5,2,4,2]<-rlh*(1-smpos_prop_h)

j1 <- array(c(1:5,rep(1,5),1:5,rep(2,5)),dim=c(5,4)) 
mastermat[j1]<-hiv_inc                                       

mastername<-paste("M",1:100,sep="")  # 10x10 parameters
mastervect <-c(mastermat)
names(mastervect)<-mastername
mastervect<-c(mastervect,extrapars)

outm28 <- ode(statevect, times2, TBdx, mastervect)
outmatm28[abc,] <- outgen(outm28[2,2:11],mastervect)
}


# Quick check that output is looking right:
fitvect <- rep(0,7)
for (abcd in 1:7) {
fitvect[abcd] <-sum(outmatm28[701:800,abcd])/100
}

fitvect



##################################################
# now bring to end 2012.
# most of this code simply continues trends from above.

for (abc in 801:1000) { 
 statevect <-(as.numeric(outm28[2,2:11]))
 statename<-paste("S",1:10,sep="")
 names(statevect)<-statename

extrapars <-c(TP = 100000,
              B_a = 0.00011062-Ba_fact*(abc),
		  Bx_hiv = max(1,3.671-hivtb_fact*abc),
              Bx_smn = 0.15,
              LP = 0.50)

total_pop <- 100000
pop_inc <-0.0225
transmit <- 0.00011062-Ba_fact*(abc)
trans_hiv <- max(1,3.671-hivtb_fact*abc)
hiv_inc <- hiv_inc2-hiv_fact*abc
init_def <- 0.15
dx_rate <- 0.80+dx_fact*abc
dx_rate_h <- 18.07-dxh_fact*abc

tx_sp <- dx_rate*tx_suc*(sm_sens*(1-init_def)+ 
         (1-sm_sens*(1-init_def))*empiric_sens)
txh_sp <- dx_rate_h*tx_suc*(sm_sens*(1-init_def)+ 
         (1-sm_sens*(1-init_def))*empiric_sens)
tx_sn <- dx_rate*tx_suc*empiric_sens
txh_sn <- dx_rate_h*tx_suc*empiric_sens


mortmat <- array(0, dim=c(5,2))
entrymat <- array(0, dim=c(5,2))
mastermat <- array(0, dim=c(5,2,5,2))
mortmat[,] <- -m_notb
mortmat[,2] <- mortmat[,2] - m_h
mortmat[3,1] <- mortmat[3,1] - m_tb_sp
mortmat[4,1] <- mortmat[4,1] - m_tb_sn
mortmat[3,2] <- mortmat[3,2] - m_tbh
mortmat[4,2] <- mortmat[4,2] - m_tbh

mortname<-paste("m",1:10,sep="")
mortvect <-c(mortmat)
names(mortvect)<-mortname

i <- rep(1:2,each=5)
j <- rep(1:5,2)
k <- array(c(j,i),dim=c(10,2))
m <- array(c(k,k),dim=c(10,4))
mastermat[m]<-mastermat[m]+mortvect
mastermat[2,1,3,1]<-lpslow*smpos_prop
mastermat[2,2,3,2]<-lpslow_h*smpos_prop_h
mastermat[2,1,4,1]<-lpslow*(1-smpos_prop)
mastermat[2,2,4,2]<-lpslow_h*(1-smpos_prop_h)
mastermat[3,1,5,1]<-tx_sp + cure_sp
mastermat[3,2,5,2]<-txh_sp
mastermat[4,1,5,1]<-tx_sn + cure_sn
mastermat[4,2,5,2]<-txh_sn
mastermat[5,1,3,1]<-rl*smpos_prop
mastermat[5,1,4,1]<-rl*(1-smpos_prop)
mastermat[5,2,3,2]<-rlh*smpos_prop_h
mastermat[5,2,4,2]<-rlh*(1-smpos_prop_h)

j1 <- array(c(1:5,rep(1,5),1:5,rep(2,5)),dim=c(5,4)) 
mastermat[j1]<-hiv_inc                                       

mastername<-paste("M",1:100,sep="")
mastervect <-c(mastermat)
names(mastervect)<-mastername
mastervect<-c(mastervect,extrapars)

outm28 <- ode(statevect, times2, TBdx, mastervect)
outmatm28[abc,] <- outgen(outm28[2,2:11],mastervect)
}

# Create a vector that can serve as the starting population for the next 10 years
out2012m28<-rep(0,10)
out2012m28<-outm28[2,2:11]

# now project 10 years under the baseline condition
# again, this simply continues trends established in 2003-2010
for (abc in 1001:2000) { 

 statevect <-(as.numeric(outm28[2,2:11]))
 statename<-paste("S",1:10,sep="")
 names(statevect)<-statename

extrapars <-c(TP = 100000,
              B_a = 0.00011062-Ba_fact*(abc),
		  Bx_hiv = max(1,3.671-hivtb_fact*abc),
              Bx_smn = 0.15,
              LP = 0.50)

total_pop <- 100000
pop_inc <-0.0225
transmit <- 0.00011062-Ba_fact*(abc)
trans_hiv <- max(1,3.671-hivtb_fact*abc)
hiv_inc <- hiv_inc2-hiv_fact*abc
init_def <- 0.15
dx_rate <- 0.80+dx_fact*abc
dx_rate_h <- 18.07-dxh_fact*abc

tx_sp <- dx_rate*tx_suc*(sm_sens*(1-init_def)+ 
         (1-sm_sens*(1-init_def))*empiric_sens)
txh_sp <- dx_rate_h*tx_suc*(sm_sens*(1-init_def)+ 
         (1-sm_sens*(1-init_def))*empiric_sens)
tx_sn <- dx_rate*tx_suc*empiric_sens
txh_sn <- dx_rate_h*tx_suc*empiric_sens


mortmat <- array(0, dim=c(5,2))
entrymat <- array(0, dim=c(5,2))
mastermat <- array(0, dim=c(5,2,5,2))
mortmat[,] <- -m_notb
mortmat[,2] <- mortmat[,2] - m_h
mortmat[3,1] <- mortmat[3,1] - m_tb_sp
mortmat[4,1] <- mortmat[4,1] - m_tb_sn
mortmat[3,2] <- mortmat[3,2] - m_tbh
mortmat[4,2] <- mortmat[4,2] - m_tbh

mortname<-paste("m",1:10,sep="")
mortvect <-c(mortmat)
names(mortvect)<-mortname

i <- rep(1:2,each=5)
j <- rep(1:5,2)
k <- array(c(j,i),dim=c(10,2))
m <- array(c(k,k),dim=c(10,4))
mastermat[m]<-mastermat[m]+mortvect
mastermat[2,1,3,1]<-lpslow*smpos_prop
mastermat[2,2,3,2]<-lpslow_h*smpos_prop_h
mastermat[2,1,4,1]<-lpslow*(1-smpos_prop)
mastermat[2,2,4,2]<-lpslow_h*(1-smpos_prop_h)
mastermat[3,1,5,1]<-tx_sp + cure_sp
mastermat[3,2,5,2]<-txh_sp
mastermat[4,1,5,1]<-tx_sn + cure_sn
mastermat[4,2,5,2]<-txh_sn
mastermat[5,1,3,1]<-rl*smpos_prop
mastermat[5,1,4,1]<-rl*(1-smpos_prop)
mastermat[5,2,3,2]<-rlh*smpos_prop_h
mastermat[5,2,4,2]<-rlh*(1-smpos_prop_h)

j1 <- array(c(1:5,rep(1,5),1:5,rep(2,5)),dim=c(5,4)) 
mastermat[j1]<-hiv_inc                                       

mastername<-paste("M",1:100,sep="")
mastervect <-c(mastermat)
names(mastervect)<-mastername
mastervect<-c(mastervect,extrapars)

outm28 <- ode(statevect, times2, TBdx, mastervect)
outmatm28[abc,] <- outgen(outm28[2,2:11],mastervect)
}

# create year 2022 population under the baseline condition (for calculation)
out2022bl<-rep(0,10)
out2022bl<-outm28[2,2:11]



# now project 10 years under the selected intervention:
outmatint[1:1000,] <-outmatm28[1:1000,] # first 10 years are the same
outm28[2,2:11] <-out2012m28 # starting population for 2012

# vector for calculating number of Xpert diagnoses:
xpertv<-rep(0,1000)

for (abc in 1001:2000) { 

 statevect <-(as.numeric(outm28[2,2:11]))
 statename<-paste("S",1:10,sep="")
 names(statevect)<-statename

extrapars <-c(TP = 100000,
              B_a = 0.00011062-Ba_fact*(abc),
		  Bx_hiv = max(1,3.671-hivtb_fact*abc),
              Bx_smn = 0.15,
              LP = 0.50)

total_pop <- 100000
pop_inc <-0.0225
transmit <- 0.00011062-Ba_fact*(abc)
trans_hiv <- max(1,3.671-hivtb_fact*abc)
hiv_inc <- hiv_inc2-hiv_fact*abc
# For same-day interventions, reduce the rate of initial default below
# Baseline is 0.15
# Below is the code for phase-in of same-day diagnosis
init_def <- max(0.015,min(0.15,0.15-(abc-1000)*0.15/200))
# For Xpert MTB/RIF, increase the rate of smear-negative diagnosis
# Baseline is 0.254
# Below is the code for phase-in of Xpert
# Thus, the code as currently written gives results for the "same-day smear + Xpert" scenario
empiric_sensn <- 0.254 + ((1-0.254) * 0.72 * (1-0.15)* max(0,min((abc-1000)*0.75/200,0.75)))
dx_rate <- 0.80+dx_fact*abc
dx_rate_h <- 18.07-dxh_fact*abc

tx_sp <- dx_rate*tx_suc*(sm_sens*(1-init_def)+ 
         (1-sm_sens*(1-init_def))*empiric_sens)
txh_sp <- dx_rate_h*tx_suc*(sm_sens*(1-init_def)+ 
         (1-sm_sens*(1-init_def))*empiric_sens)
tx_sn <- dx_rate*tx_suc*empiric_sensn
txh_sn <- dx_rate_h*tx_suc*empiric_sensn

mortmat <- array(0, dim=c(5,2))
entrymat <- array(0, dim=c(5,2))
mastermat <- array(0, dim=c(5,2,5,2))
mortmat[,] <- -m_notb
mortmat[,2] <- mortmat[,2] - m_h
mortmat[3,1] <- mortmat[3,1] - m_tb_sp
mortmat[4,1] <- mortmat[4,1] - m_tb_sn
mortmat[3,2] <- mortmat[3,2] - m_tbh
mortmat[4,2] <- mortmat[4,2] - m_tbh

mortname<-paste("m",1:10,sep="")
mortvect <-c(mortmat)
names(mortvect)<-mortname

i <- rep(1:2,each=5)
j <- rep(1:5,2)
k <- array(c(j,i),dim=c(10,2))
m <- array(c(k,k),dim=c(10,4))
mastermat[m]<-mastermat[m]+mortvect
mastermat[2,1,3,1]<-lpslow*smpos_prop
mastermat[2,2,3,2]<-lpslow_h*smpos_prop_h
mastermat[2,1,4,1]<-lpslow*(1-smpos_prop)
mastermat[2,2,4,2]<-lpslow_h*(1-smpos_prop_h)
mastermat[3,1,5,1]<-tx_sp + cure_sp
mastermat[3,2,5,2]<-txh_sp
mastermat[4,1,5,1]<-tx_sn + cure_sn
mastermat[4,2,5,2]<-txh_sn
mastermat[5,1,3,1]<-rl*smpos_prop
mastermat[5,1,4,1]<-rl*(1-smpos_prop)
mastermat[5,2,3,2]<-rlh*smpos_prop_h
mastermat[5,2,4,2]<-rlh*(1-smpos_prop_h)

j1 <- array(c(1:5,rep(1,5),1:5,rep(2,5)),dim=c(5,4)) 
mastermat[j1]<-hiv_inc                                       

mastername<-paste("M",1:100,sep="")
mastervect <-c(mastermat)
names(mastervect)<-mastername
mastervect<-c(mastervect,extrapars)

outm28 <- ode(statevect, times2, TBdx, mastervect)
outmatint[abc,] <- outgen(outm28[2,2:11],mastervect)
xpertv[abc-1000] <- (dx_rate*sm_sens*outm28[2,4]+dx_rate*0.72*outm28[2,5]+
   dx_rate_h*sm_sens*outm28[2,9]+dx_rate_h*0.72*outm28[2,10]) * 
   max(0,min((abc-1000)*0.75/200,0.75))/100
}


# print the end of 2012 under baseline vs. intervention:
outmatm28[1000,]
outmatint[1000,]

# print the end of 2022 under baseline vs. intervention:
outmatm28[2000,]
outmatint[2000,]

# print the total number of true-positive Xpert diagnoses
sum(xpertv)

# output the calculations as a CSV file
# Column 1 is population (scaled to full population of Africa)
# Column 2 is TB incidence
# Column 3 is HIV/TB incidence
# Column 4 is TB prevalence
# Column 5 is HIV prevalence
# Column 6 is TB mortality
# Column 7 is HIV/TB mortality
# Each row is a timestep (2000 timesteps = 20 years from 2002 to 2022)
write.csv(outmatint,file="file_int.csv")
